@@ -22,7 +22,7 @@ http://www.decalage.info/python/oletools
 
 # === LICENSE ==================================================================
 
-# MacroRaptor is copyright (c) 2016 Philippe Lagadec (http://www.decalage.info)
+# MacroRaptor is copyright (c) 2016-2017 Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -54,8 +54,10 @@ http://www.decalage.info/python/oletools
 # 2016-08-31 v0.50 PL: - added macro trigger InkPicture_Painted
 # 2016-09-05       PL: - added Document_BeforeClose keyword for MS Publisher (.pub)
 # 2016-10-25       PL: - fixed print for Python 3
+# 2016-12-21 v0.51 PL: - added more ActiveX macro triggers
+# 2017-03-08       PL: - fixed absolute imports
 
-__version__ = '0.50'
+__version__ = '0.51'
 
 #------------------------------------------------------------------------------
 # TODO:
@@ -63,12 +65,24 @@ __version__ = '0.50'
 
 #--- IMPORTS ------------------------------------------------------------------
 
-import sys, logging, optparse, re
+import sys, logging, optparse, re, os
 
-from thirdparty.xglob import xglob
-from thirdparty.tablestream import tablestream
+# IMPORTANT: it should be possible to run oletools directly as scripts
+# in any directory without installing them with pip or setup.py.
+# In that case, relative imports are NOT usable.
+# And to enable Python 2+3 compatibility, we need to use absolute imports,
+# so we add the oletools parent folder to sys.path (absolute+normalized path):
+_thismodule_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
+# print('_thismodule_dir = %r' % _thismodule_dir)
+_parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
+# print('_parent_dir = %r' % _thirdparty_dir)
+if not _parent_dir in sys.path:
+    sys.path.insert(0, _parent_dir)
 
-import olevba
+from oletools.thirdparty.xglob import xglob
+from oletools.thirdparty.tablestream import tablestream
+
+from oletools import olevba
 
 # === LOGGING =================================================================
 
@@ -91,12 +105,18 @@ MSG_ISSUES = 'Please report this issue on %s' % URL_ISSUES
 # 'Workbook_Open', 'Workbook_Close',
 # *_Painted such as InkPicture1_Painted
 # *_GotFocus|LostFocus|MouseHover for other ActiveX objects
+# reference: http://www.greyhathacker.net/?p=948
 
 # TODO: check if line also contains Sub or Function
 re_autoexec = re.compile(r'(?i)\b(?:Auto(?:Exec|_?Open|_?Close|Exit|New)' +
                          r'|Document(?:_?Open|_Close|_?BeforeClose|Change|_New)' +
                          r'|NewDocument|Workbook(?:_Open|_Activate|_Close)' +
-                         r'|\w+_(?:Painted|GotFocus|LostFocus|MouseHover))\b')
+                         r'|\w+_(?:Painted|Painting|GotFocus|LostFocus|MouseHover' +
+                         r'|Layout|Click|Change|Resize|BeforeNavigate2|BeforeScriptExecute' +
+                         r'|DocumentComplete|DownloadBegin|DownloadComplete|FileDownload' +
+                         r'|NavigateComplete2|NavigateError|ProgressChange|PropertyChange' +
+                         r'|SetSecureLockIcon|StatusTextChange|TitleChange|MouseMove' +
+                         r'|MouseEnter|MouseLeave|))\b')
 
 # MS-VBAL 5.4.5.1 Open Statement:
 RE_OPEN_WRITE = r'(?:\bOpen\b[^\n]+\b(?:Write|Append|Binary|Output|Random)\b)'
@@ -221,7 +241,7 @@ def main():
         'critical': logging.CRITICAL
         }
 
-    usage = 'usage: %prog [options] <filename> [filename2 ...]'
+    usage = 'usage: mraptor [options] <filename> [filename2 ...]'
     parser = optparse.OptionParser(usage=usage)
     parser.add_option("-r", action="store_true", dest="recursive",
                       help='find files recursively in subdirectories.')
@@ -240,6 +260,8 @@ def main():
 
     # Print help if no arguments are passed
     if len(args) == 0:
+        print('MacroRaptor %s - http://decalage.info/python/oletools' % __version__)
+        print('This is work in progress, please report issues at %s' % URL_ISSUES)
         print(__doc__)
         parser.print_help()
         print('\nAn exit code is returned based on the analysis result:')
