@@ -18,7 +18,7 @@ http://www.decalage.info/python/oletools
 
 #=== LICENSE =================================================================
 
-# oleid is copyright (c) 2012-2016, Philippe Lagadec (http://www.decalage.info)
+# oleid is copyright (c) 2012-2017, Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -41,7 +41,9 @@ http://www.decalage.info/python/oletools
 # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
+# To improve Python 2+3 compatibility:
 from __future__ import print_function
+#from __future__ import absolute_import
 
 #------------------------------------------------------------------------------
 # CHANGELOG:
@@ -50,8 +52,10 @@ from __future__ import print_function
 #                      - improved usage display with -h
 # 2014-11-30 v0.03 PL: - improved output with prettytable
 # 2016-10-25 v0.50 PL: - fixed print and bytes strings for Python 3
+# 2016-12-12 v0.51 PL: - fixed relative imports for Python 3 (issue #115)
+# 2017-04-26       PL: - fixed absolute imports (issue #141)
 
-__version__ = '0.50'
+__version__ = '0.51'
 
 
 #------------------------------------------------------------------------------
@@ -73,8 +77,22 @@ __version__ = '0.50'
 #=== IMPORTS =================================================================
 
 import optparse, sys, os, re, zlib, struct
-import thirdparty.olefile as olefile
-from thirdparty.prettytable import prettytable
+
+# IMPORTANT: it should be possible to run oletools directly as scripts
+# in any directory without installing them with pip or setup.py.
+# In that case, relative imports are NOT usable.
+# And to enable Python 2+3 compatibility, we need to use absolute imports,
+# so we add the oletools parent folder to sys.path (absolute+normalized path):
+_thismodule_dir = os.path.normpath(os.path.abspath(os.path.dirname(__file__)))
+# print('_thismodule_dir = %r' % _thismodule_dir)
+_parent_dir = os.path.normpath(os.path.join(_thismodule_dir, '..'))
+# print('_parent_dir = %r' % _thirdparty_dir)
+if not _parent_dir in sys.path:
+    sys.path.insert(0, _parent_dir)
+
+from oletools.thirdparty import olefile
+from oletools.thirdparty.prettytable import prettytable
+
 
 
 #=== FUNCTIONS ===============================================================
@@ -265,6 +283,12 @@ class OleID:
 #=== MAIN =================================================================
 
 def main():
+    # print banner with version
+    print ('oleid %s - http://decalage.info/oletools' % __version__)
+    print ('THIS IS WORK IN PROGRESS - Check updates regularly!')
+    print ('Please report any issue at https://github.com/decalage2/oletools/issues')
+    print ('')
+
     usage = 'usage: %prog [options] <file>'
     parser = optparse.OptionParser(usage=__doc__ + '\n' + usage)
 ##    parser.add_option('-o', '--ole', action='store_true', dest='ole', help='Parse an OLE file (e.g. Word, Excel) to look for SWF in each stream')
@@ -277,7 +301,7 @@ def main():
         return
 
     for filename in args:
-        print('\nFilename:', filename)
+        print('Filename:', filename)
         oleid = OleID(filename)
         indicators = oleid.check()
 
@@ -293,6 +317,7 @@ def main():
             t.add_row((indicator.name, indicator.value))
 
         print(t)
+        print ('')
 
 if __name__ == '__main__':
     main()
