@@ -17,7 +17,7 @@ http://www.decalage.info/python/oletools
 
 #=== LICENSE =================================================================
 
-# rtfobj is copyright (c) 2012-2017, Philippe Lagadec (http://www.decalage.info)
+# rtfobj is copyright (c) 2012-2018, Philippe Lagadec (http://www.decalage.info)
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without modification,
@@ -73,8 +73,14 @@ http://www.decalage.info/python/oletools
 # 2017-05-04       PL: - fixed issue #164 to handle linked OLE objects
 # 2017-06-08       PL: - fixed issue/PR #143: bin object with negative length
 # 2017-06-29       PL: - temporary fix for issue #178
+# 2017-07-14 v0.52 PL: - disabled logging of each control word (issue #184)
+# 2017-07-24       PL: - fixed call to RtfParser._end_of_file (issue #185)
+#                      - ignore optional space after \bin (issue #185)
+# 2017-09-06       PL: - fixed issue #196: \pxe is not a destination
+# 2018-01-11       CH: - speedup RTF parsing (PR #244)
+# 2018-02-01      JRM: - fixed issue #251: \bin without argument
 
-__version__ = '0.51'
+__version__ = '0.52'
 
 # ------------------------------------------------------------------------------
 # TODO:
@@ -90,6 +96,7 @@ __version__ = '0.51'
 
 import re, os, sys, binascii, logging, optparse
 import os.path
+from time import time
 
 # IMPORTANT: it should be possible to run oletools directly as scripts
 # in any directory without installing them with pip or setup.py.
@@ -105,7 +112,6 @@ if not _parent_dir in sys.path:
 
 from oletools.thirdparty.xglob import xglob
 from oletools.thirdparty.tablestream import tablestream
-from oletools.oleobj import OleObject, OleNativeStream
 from oletools import oleobj
 
 # === LOGGING =================================================================
@@ -282,8 +288,9 @@ DESTINATION_CONTROL_WORDS = frozenset((
     b"oleclsid", b"operator", b"panose", b"password", b"passwordhash", b"pgp", b"pgptbl", b"picprop", b"pict", b"pn", b"pnseclvl",
     b"pntext", b"pntxta", b"pntxtb", b"printim",
     # It seems \private should not be treated as a destination (issue #178)
-    # b"private",
-    b"propname", b"protend", b"protstart", b"protusertbl", b"pxe",
+    # Same for \pxe (issue #196)
+    # b"private", b"pxe",
+    b"propname", b"protend", b"protstart", b"protusertbl",
     b"result", b"revtbl", b"revtim", b"rsidtbl", b"rtf", b"rxe", b"shp", b"shpgrp", b"shpinst", b"shppict", b"shprslt", b"shptxt",
     b"sn", b"sp", b"staticval", b"stylesheet", b"subject", b"sv", b"svb", b"tc", b"template", b"themedata", b"title", b"txe", b"ud",
     b"upr", b"userprops", b"wgrffmtfilter", b"windowcaption", b"writereservation", b"writereservhash", b"xe", b"xform",
@@ -298,11 +305,31 @@ if sys.version_info[0] <= 2:
     BACKSLASH = '\\'
     BRACE_OPEN = '{'
     BRACE_CLOSE = '}'
+    UNICODE_TYPE = unicode
 else:
     # Python 3.x - Integers
     BACKSLASH = ord('\\')
     BRACE_OPEN = ord('{')
     BRACE_CLOSE = ord('}')
+    UNICODE_TYPE = str
+
+RTF_MAGIC = b'\x7b\\rt'   # \x7b == b'{' but does not mess up auto-indent
+
+
+def duration_str(duration):
+    """ create a human-readable string representation of duration [s] """
+    value = duration
+    unit = 's'
+    if value > 90:
+        value /= 60.
+        unit = 'min'
+        if value > 90:
+            value /= 60.
+            unit = 'h'
+            if value > 72:
+                value /= 24.
+                unit = 'days'
+    return '{0:.1f}{1}'.format(value, unit)
 
 
 #=== CLASSES =================================================================
@@ -351,6 +378,20 @@ class RtfParser(object):
         self.destinations = [document_destination]
         self.current_destination = document_destination
 
+    def _report_progress(self, start_time):
+        """ report progress on parsing at regular intervals """
+        now = float(time())
+        if now == start_time or self.size == 0:
+            return   # avoid zero-division
+        percent_done = 100. * self.index / self.size
+        time_per_index = (now - start_time) / float(self.index)
+        finish_estim = float(self.size - self.index) * time_per_index
+
+        log.debug('After {0} finished {1:4.1f}% of current file ({2} bytes); '
+                  'will finish in approx {3}'
+                  .format(duration_str(now-start_time), percent_done,
+                          self.size, duration_str(finish_estim)))
+
     def parse(self):
         """
         Parse the RTF data
@@ -359,8 +400,13 @@ class RtfParser(object):
         """
         # Start at beginning of data
         self.index = 0
+        start_time = time()
+        last_report = start_time
         # Loop until the end
         while self.index < self.size:
+            if time() - last_report > 15:     # report every 15s
+                self._report_progress(start_time)
+                last_report = time()
             if self.data[self.index] == BRACE_OPEN:
                 # Found an opening brace "{": Start of a group
                 self._open_group()
@@ -377,9 +423,7 @@ class RtfParser(object):
                 # NOTE: the full length of the control word + its optional integer parameter
                 # is limited by MS Word at 253 characters, so we have to run the regex
                 # on a cropped string:
-                data_cropped = self.data[self.index:]
-                if len(data_cropped)>253:
-                    data_cropped = data_cropped[:254]
+                data_cropped = self.data[self.index:self.index+254]
                 # append a space so that the regex can check the following character:
                 data_cropped += b' '
                 # m = re_control_word.match(self.data, self.index, self.index+253)
@@ -389,7 +433,7 @@ class RtfParser(object):
                     param = None
                     if len(m.groups()) > 1:
                         param = m.group(2)
-                    # log.debug('control word %r at index %Xh - cword=%r param=%r' % (m.group(), self.index, cword, param))
+                    # log.debug('control word at index %Xh - cword=%r param=%r  %r' % (self.index, cword, param, m.group()))
                     self._control_word(m, cword, param)
                     self.index += len(m.group())
                     # if it's \bin, call _bin after updating index
@@ -410,7 +454,8 @@ class RtfParser(object):
                 self.index += len(m.group())
                 continue
             raise RuntimeError('Should not have reached this point - index=%Xh' % self.index)
-        self.end_of_file()
+        # call _end_of_file to make sure all groups are closed properly
+        self._end_of_file()
 
 
     def _open_group(self):
@@ -505,16 +550,24 @@ class RtfParser(object):
         pass
 
     def _bin(self, matchobject, param):
-        binlen = int(param)
+        if param is None:
+            log.info('Detected anti-analysis trick: \\bin object without length at index %X' % self.index)
+            binlen = 0
+        else:
+            binlen = int(param)
+        # handle negative length
         if binlen < 0:
-            log.warn('Detected anti-analysis trick: \\bin object with negative length at index %X' % self.index)
+            log.info('Detected anti-analysis trick: \\bin object with negative length at index %X' % self.index)
             # binlen = int(param.strip('-'))
             # According to my tests, if the bin length is negative,
             # it should be treated as a null length:
             binlen=0
+        # ignore optional space after \bin
+        if self.data[self.index] == ' ':
+            log.debug('\\bin: ignoring whitespace before data')
+            self.index += 1
         log.debug('\\bin: reading %d bytes of binary data' % binlen)
-        # TODO: handle optional space?
-        # TODO: handle negative length, and length greater than data
+        # TODO: handle length greater than data
         bindata = self.data[self.index:self.index + binlen]
         self.index += binlen
         self.bin(bindata)
@@ -526,7 +579,7 @@ class RtfParser(object):
         # log.debug('%Xh Reached End of File')
         # close any group/destination that is still open:
         while self.group_level > 0:
-            # log.debug('Group Level = %d, closing group' % self.group_level)
+            log.debug('Group Level = %d, closing group' % self.group_level)
             self._close_group()
         self.end_of_file()
 
@@ -603,7 +656,7 @@ class RtfObjParser(RtfParser):
             rtfobj.rawdata = object_data
             # TODO: check if all hex data is extracted properly
 
-            obj = OleObject()
+            obj = oleobj.OleObject()
             try:
                 obj.parse(object_data)
                 rtfobj.format_id = obj.format_id
@@ -612,7 +665,8 @@ class RtfObjParser(RtfParser):
                 rtfobj.oledata = obj.data
                 rtfobj.is_ole = True
                 if obj.class_name.lower() == 'package':
-                    opkg = OleNativeStream(bindata=obj.data, package=True)
+                    opkg = oleobj.OleNativeStream(bindata=obj.data,
+                                                  package=True)
                     rtfobj.filename = opkg.filename
                     rtfobj.src_path = opkg.src_path
                     rtfobj.temp_path = opkg.temp_path
@@ -632,7 +686,7 @@ class RtfObjParser(RtfParser):
         # TODO: extract useful cwords such as objclass
         # TODO: keep track of cwords inside objdata, because it is unusual and indicates potential obfuscation
         # TODO: same with control symbols, and opening bracket
-        log.debug('- Control word "%s", param=%s, level=%d' % (cword, param, self.group_level))
+        # log.debug('- Control word "%s", param=%s, level=%d' % (cword, param, self.group_level))
         pass
 
 
@@ -658,7 +712,50 @@ def rtf_iter_objects(filename, min_size=32):
         yield obj.start, orig_len, obj.rawdata
 
 
+def is_rtf(arg, treat_str_as_data=False):
+    """ determine whether given file / stream / array represents an rtf file
 
+    arg can be either a file name, a byte stream (located at start), a
+    list/tuple or a an iterable that contains bytes.
+
+    For str it is not clear whether data is a file name or the data read from
+    it (at least for py2-str which is bytes). Argument treat_str_as_data
+    clarifies.
+    """
+    magic_len = len(RTF_MAGIC)
+    if isinstance(arg, UNICODE_TYPE):
+        with open(arg, 'rb') as reader:
+            return reader.read(len(RTF_MAGIC)) == RTF_MAGIC
+    if isinstance(arg, bytes) and not isinstance(arg, str):  # only in PY3
+        return arg[:magic_len] == RTF_MAGIC
+    if isinstance(arg, bytearray):
+        return arg[:magic_len] == RTF_MAGIC
+    if isinstance(arg, str):      # could be bytes, but we assume file name
+        if treat_str_as_data:
+            try:
+                return arg[:magic_len].encode('ascii', errors='strict')\
+                    == RTF_MAGIC
+            except UnicodeError:
+                return False
+        else:
+            with open(arg, 'rb') as reader:
+                return reader.read(len(RTF_MAGIC)) == RTF_MAGIC
+    if hasattr(arg, 'read'):      # a stream (i.e. file-like object)
+        return arg.read(len(RTF_MAGIC)) == RTF_MAGIC
+    if isinstance(arg, (list, tuple)):
+        iter_arg = iter(arg)
+    else:
+        iter_arg = arg
+
+    # check iterable
+    for magic_byte in zip(RTF_MAGIC):
+        try:
+            if next(iter_arg) not in magic_byte:
+                return False
+        except StopIteration:
+            return False
+
+    return True  # checked the complete magic without returning False --> match
 
 
 def sanitize_filename(filename, replacement='_', max_length=200):
